<?php
/*--------------------------------------------------------------
   product_variants_migration.php 2022-07-11
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2022 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
 -------------------------------------------------------------*/

/** @var DatabaseModel $this */
/** @var int $t_success */

$updateWasExecutedQuery  = 'SELECT * FROM `gx_configurations` WHERE `key` = "gambio_updater/v4.5.1.0/variant_migration"';
$updateWasExecutedResult = $this->query($updateWasExecutedQuery, true);

$displayImageUpdateQuery = "UPDATE `products_options_values` `pov` SET `pov`.`display_image`=`pov`.`gm_filename` WHERE `pov`.`gm_filename`!=''";
$t_success               &= $this->query($displayImageUpdateQuery, true);

if ($updateWasExecutedResult->num_rows < 1) {
    
    //  =====================================
    //  migrating property & attribute
    //  images to new option directory
    //  =====================================
    $shopRoot        = dirname(__DIR__, 3);
    $productImages   = $shopRoot . '/images/product_images';
    $attributeImages = $productImages . '/attribute_images';
    $propertyImages  = $productImages . '/property_images';
    $optionImages    = $shopRoot . '/cache/option_images';
    
    if (is_dir($optionImages) === false) {
        
        mkdir($optionImages) or die(sprintf('Could not create directory %s', $optionImages));
        @chmod($optionImages, 0777);
    }
    
    /**
     * @param string   $path                         path from where the images should be copied
     * @param string   $destinationDirectory         where the files should be copied to
     * @param string   $originalDirectory            must be same as $path at the initial function call, for recursion purposes
     * @param callable $recursivePropertyImageCopier declared function so it can be called recursively
     */
    $recursiveImageCopier = function (
        string $path,
        string $destinationDirectory,
        string $originalDirectory,
        callable $recursivePropertyImageCopier
    ): void {
        
        foreach (new DirectoryIterator($path) as $file) {
            
            /** @var DirectoryIterator $file */
            if ($file->isDot() || $file->getExtension() === 'html') {
                
                continue;
            }
            
            $absolutePath = $path . '/' . $file->getFilename();
            
            if ($file->isDir()) {
    
                $recursivePropertyImageCopier($absolutePath, $destinationDirectory, $originalDirectory, $recursivePropertyImageCopier);
                continue;
            }
            
            $destination = preg_replace('/^' . preg_quote($originalDirectory, '/') . '/', $destinationDirectory, $absolutePath);
            
            if (is_dir(dirname($destination)) === false) {
                
                mkdir(dirname($destination), 0777, true) or die(sprintf('Could not create directory %s', dirname($destination)));
                @chmod(dirname($destination), 0777);
            }
    
            while (file_exists($destination) === true) {
        
                $destination = preg_replace('/(^.*\/)([^\/]*)$/', '$1options_$2', $destination);
            }
            
            copy($absolutePath, $destination);
            @chmod($destination, 0777);
        }
    };
    
    //  copying property images to option images
    if (is_dir($propertyImages)) {
        
        $recursiveImageCopier($propertyImages, $optionImages, $propertyImages, $recursiveImageCopier);
    }
    
    //  copying attribute images to option images
    if (is_dir($attributeImages)) {
        
        $recursiveImageCopier($attributeImages, $optionImages, $attributeImages, $recursiveImageCopier);
    }
    
    //  copying option images to (cache) option images
    if (is_dir($productImages . '/option_images')) {
        
        $recursiveImageCopier($productImages . '/option_images', $optionImages, $productImages . '/option_images', $recursiveImageCopier);
    }
    
    //  =====================================
    //  copying properties to attributes
    //  =====================================
    $availablePropertiesQuery = '
        SELECT
            `p`.`properties_id`, `p`.`sort_order`, `p`.`display_type`,
            `pd`. `properties_description_id`, `pd`.`language_id`, `pd`.`properties_name`, `pd`.`properties_admin_name`, `pd`.`description`
        FROM `properties_description` as `pd`
        INNER JOIN `properties` as `p` ON `pd`.`properties_id` = `p`.`properties_id`';

    $availableProperties = $this->query($availablePropertiesQuery, true);
    $properties          = [];

    while ($row = $availableProperties->fetch_assoc()) {
    
        $properties[(int)$row['properties_id']][(int)$row['language_id']] = $row;
    }
    
    if (count($properties)) {

        $propertyIds = implode(', ', array_keys($properties));
    
        $availablePropertyValuesQuery = "
            SELECT
                pv.`properties_values_id`, pv.`properties_id`, pv.`sort_order`, pv.`value_model`, pv.`weight`,
                pv.`value_price`, pv.`display_image`, pv.`stock_centrally_managed`, pv.`stock`, pv.`stock_type`,
                pvd.`properties_values_description_id`, pvd.`language_id`, pvd.`values_name`, pvd.`description`
            FROM `properties_values` as pv
            INNER JOIN `properties_values_description` as pvd
            ON pv.`properties_values_id` = pvd.`properties_values_id`
            WHERE pv.`properties_id` IN ($propertyIds)";
    
        $availablePropertyValues = $this->query($availablePropertyValuesQuery, true);
        $propertyValues          = [];
    
        while ($row = $availablePropertyValues->fetch_assoc()) {
        
            $propertyId      = (int)$row['properties_id'];
            $propertyValueId = (int)$row['properties_values_id'];
            $languageId      = (int)$row['language_id'];
        
            $propertyValues[$propertyId][$propertyValueId][$languageId] = $row;
        }
    
        $nextProductOptionId = static function (DatabaseModel $dm): int {
        
            $query  = "SELECT MAX(`products_options_id`) AS 'max_poi' FROM `products_options`";
            $result = $dm->query($query, true);
        
            return (int)$result->fetch_assoc()['max_poi'] + 1;
        };
    
        $nextProductsOptionsValuesId = static function (DatabaseModel $dm): int {
        
            $query  = "SELECT MAX(`products_options_values_id`) AS 'max_povi' FROM `products_options_values`";
            $result = $dm->query($query, true);
        
            return (int)$result->fetch_assoc()['max_povi'] + 1;
        };
    
        foreach ($properties as $propertyId => $localizations) {
        
            $productOptionId          = $nextProductOptionId($this);
            $insertProductOptionQuery = "
                INSERT INTO `products_options`
                    (`products_options_id`, `language_id`, `options_id`, `products_options_name`, `products_option_display_type`, `sort_order`, `description`, `admin_label`)
                VALUES ";
        
            foreach ($localizations as $localization) {
            
                $sortOrder   = (int)$localization['sort_order'];
                $languageId  = (int)$localization['language_id'];
                $displayType = $this->real_escape_string($localization['display_type']);
                $name        = $this->real_escape_string($localization['properties_name']);
                $adminName   = $this->real_escape_string($localization['properties_admin_name']);
                $description = $this->real_escape_string($localization['description']);
            
                $insertProductOptionQuery .= "
                ($productOptionId, $languageId, $propertyId, '$name', '$displayType', $sortOrder, '$description', '$adminName'), ";
            }
        
            $insertProductOptionQuery = substr($insertProductOptionQuery, 0, -2) . ';';
        
            $t_success &= $this->query($insertProductOptionQuery, true);
        
            $insertProductOptionValueQuery  = '
                INSERT INTO `products_options_values`
                    (
                        `products_options_values_id`, `language_id`, `products_options_values_name`, `gm_filename`,
                        `stock_centrally_managed`, `stock`, `stock_type`, `display_image`, `value_price`, `weight`,
                        `value_model`, `sort_order`, `description`, `option_value_id`
                    )
                VALUES ';
            $insertOptionToOptionValueQuery = '
                INSERT INTO `products_options_values_to_products_options`
                    (`products_options_id`, `products_options_values_id`, `option_value_id`, `options_id`)
                VALUES ';
        
            if (count($propertyValues[$propertyId])) {
            
                $productOptionValueId = $nextProductsOptionsValuesId($this);
            
                foreach ($propertyValues[$propertyId] as $valueIdArray) {
    
                    foreach ($valueIdArray as $values) {
        
                        $languageId            = (int)$values['language_id'];
                        $valueName             = $this->real_escape_string($values['values_name']);
                        $image                 = isset($values['display_image']) ? $this->real_escape_string($values['display_image']) : '';
                        $stockCentrallyManaged = (int)$values['stock_centrally_managed'];
                        $stock                 = (float)$values['stock'];
                        $stockType             = $this->real_escape_string($values['stock_type']);
                        $price                 = (float)$values['value_price'];
                        $weight                = (float)$values['weight'];
                        $model                 = $this->real_escape_string($values['value_model']);
                        $sortOrder             = (int)$values['sort_order'];
                        $description           = $this->real_escape_string($values['description']);
                        $optionValueId         = (int)$values['properties_values_id'];
        
                        $insertProductOptionValueQuery .= "
                            ($productOptionValueId, $languageId, '$valueName', '$image', $stockCentrallyManaged, $stock, '$stockType', NULL, $price, $weight, '$model', $sortOrder, '$description', $optionValueId), ";
                    }
                
                    $insertOptionToOptionValueQuery .= "($productOptionId, " . $productOptionValueId++ . ", $optionValueId, $propertyId), ";
                }
            
                $insertProductOptionValueQuery  = substr($insertProductOptionValueQuery, 0, -2) . ';';
                $insertOptionToOptionValueQuery = substr($insertOptionToOptionValueQuery, 0, -2) . ';';
            
                $t_success &= $this->query($insertProductOptionValueQuery, true);
                $t_success &= $this->query($insertOptionToOptionValueQuery, true);
            }
        }
    }

    unset($properties, $propertyValues, $localizations);

    //  =====================================
    //  copying attributes to properties
    //  =====================================
    $availableProductOptionsQuery = '
        SELECT `products_options_id`, `language_id`, `products_options_name`, `products_option_display_type`, `sort_order`, `description`, `admin_label`
        FROM `products_options`
        WHERE `options_id` IS NULL
    ';
    
    $availableProductOptions = $this->query($availableProductOptionsQuery, true);
    $productOptions          = [];
    
    while ($row = $availableProductOptions->fetch_assoc()) {
    
        $productOptions[(int)$row['products_options_id']][(int)$row['language_id']] = $row;
    }
    
    if (count($productOptions)) {
    
        $unMigratedProductOptions          = implode(', ', array_keys($productOptions));
        $availableProductOptionValuesQuery = '
            SELECT
                povtpo.`products_options_id`, pov.`products_options_values_id`, pov.`language_id`, pov.`products_options_values_name`, pov.`gm_filename`, pov.`stock_centrally_managed`,
                pov.`stock`, pov.`stock_type`, pov.`display_image`, pov.`value_price`, pov.`weight`, pov.`value_model`, pov.`sort_order`, pov.`description`, pov.`option_value_id`
            FROM `products_options_values` AS pov
            INNER JOIN `products_options_values_to_products_options` AS povtpo ON  povtpo.`products_options_values_id` = pov.`products_options_values_id`
            WHERE povtpo.`products_options_id` IN (' . $unMigratedProductOptions . ')
        ';
    
        $availableProductOptionValues = $this->query($availableProductOptionValuesQuery, true);
        $productOptionValues          = [];
        
        while ($row = $availableProductOptionValues->fetch_assoc()) {
    
            $productsOptionsId    = (int)$row['products_options_id'];
            $productOptionValueId = (int)$row['products_options_values_id'];
            $languageId           = (int)$row['language_id'];
            
            $productOptionValues[$productsOptionsId][$productOptionValueId][$languageId] = $row;
        }
        
        foreach ($productOptions as $productsOptionsId => $localizations) {
    
            $firstAvailableLocalization = current($localizations);
            $displayType                = $this->real_escape_string($firstAvailableLocalization['products_option_display_type']);
            $sortOrder                  = $firstAvailableLocalization['sort_order']
                                          === null ? 0 : (int)$firstAvailableLocalization['sort_order'];
            
            $createPropertyQuery = "INSERT INTO `properties`(`sort_order`, `display_type`) VALUES ($sortOrder, '$displayType')";
            $t_success &= $this->query($createPropertyQuery, true);
            
            $propertyId = $this->get_insert_id();
            
            $createPropertyDescriptionQuery = '
                INSERT INTO `properties_description`(`properties_id`, `language_id`, `properties_name`, `properties_admin_name`, `description`)
                VALUES ';
            
            //  adding description values to db
            foreach ($localizations as $languageId => $localization) {
    
                $description  = $this->real_escape_string($localization['description']);
                $propertyName = $this->real_escape_string($localization['products_options_name']);
                $adminName    = $this->real_escape_string($localization['admin_label']);
                
                $createPropertyDescriptionQuery .= "($propertyId, $languageId, '$propertyName', '$adminName', '$description'), ";
            }
    
            $createPropertyDescriptionQuery = substr($createPropertyDescriptionQuery, 0, -2) . ';';
    
            $t_success &= $this->query($createPropertyDescriptionQuery, true);
            
            //  adding option id to existing attribute
            $updateProductOptionsQuery = "
                UPDATE `products_options` SET `options_id` = $propertyId WHERE `products_options_id` = $productsOptionsId";
    
            $t_success &= $this->query($updateProductOptionsQuery, true);
            
            // inserting property values
            if (count($productOptionValues[$productsOptionsId])) {
    
                foreach ($productOptionValues[$productsOptionsId] as $productOptionValueId => $productOptionValuesData) {
    
                    $firstRecord              = current($productOptionValuesData);
                    $sortOrder                = $firstRecord['sort_order']
                                                === null ? 0 : (int)$firstRecord['sort_order'];
                    $valueModel               = $this->real_escape_string($firstRecord['value_model']);
                    $weight                   = (float)$firstRecord['weight'];
                    $valuePrice               = (float)$firstRecord['value_price'];
                    $displayImage             = $this->real_escape_string($firstRecord['display_image']);
                    $stockCentrallyManaged    = $this->real_escape_string($firstRecord['stock_centrally_managed']);
                    $stock                    = (float)$firstRecord['stock'];
                    $stockType                = $this->real_escape_string($firstRecord['stock_type']);
                    $createPropertyValueQuery = "
                        INSERT INTO `properties_values`
                            (`properties_id`, `sort_order`, `value_model`, `weight`, `value_price`, `display_image`, `stock_centrally_managed`, `stock`, `stock_type`)
                        VALUES
                            ($propertyId, $sortOrder, '$valueModel', $weight, $valuePrice, '$displayImage', '$stockCentrallyManaged', $stock, '$stockType')";
    
                    $t_success &= $this->query($createPropertyValueQuery, true);
                    $propertyValueId = $this->get_insert_id();
    
                    //  inserting localized property value content
                    $createPropertyValueDescriptionQuery = "
                        INSERT INTO `properties_values_description`(`properties_values_id`, `language_id`, `values_name`, `description`) VALUES ";
                    
                    foreach ($productOptionValuesData as $languageId => $productOptionValue) {
    
                        $valueName   = $this->real_escape_string($productOptionValue['products_options_values_name']);
                        $description = $this->real_escape_string($productOptionValue['description']);
                        
                        $createPropertyValueDescriptionQuery .= "($propertyValueId, $languageId, '$valueName', '$description'), ";
                    }
    
                    $createPropertyValueDescriptionQuery = substr($createPropertyValueDescriptionQuery, 0, -2) . ';';
    
                    $t_success &= $this->query($createPropertyValueDescriptionQuery, true);
                    
                    //  setting `option_value_id` in `products_options_values` table
                    $addOptionValueIdQuery = "
                        UPDATE `products_options_values` SET `option_value_id` = $propertyValueId WHERE `products_options_values_id` = $productOptionValueId";
                    $t_success &= $this->query($addOptionValueIdQuery, true);
                    
                    //  setting `option_id` & `option_value_id` in `products_options_values_to_products_options`
                    $addOptionAndOptionValueIdQuery = "
                        UPDATE `products_options_values_to_products_options`
                        SET `option_value_id` = $propertyValueId,`options_id` = $propertyId
                        WHERE `products_options_id` = $productsOptionsId AND `products_options_values_id` = $productOptionValueId";
    
                    $t_success &= $this->query($addOptionValueIdQuery, true);
                }
            }
        }
    }
    
    //  =====================================
    //  saving config value,
    //  so this script is only executed once
    //  =====================================
    $updateSuccessfulQuery = "INSERT INTO `gx_configurations` (`key`, `value`, `default`, `type`, `sort_order`) VALUES ('gambio_updater/v4.5.1.0/variant_migration', '1', '1', NULL, NULL);";
    $t_success &= $this->query($updateSuccessfulQuery, true);
}

//  ========================================
//  Repairing previously valid option data
//  that will now cause an exception
//  ========================================
$defaultNameForEmptyEnglish = 'Unnamed option';     // is also used for every other language that is not german
$defaultNameForEmptyGerman  = 'Unbenannte Option';

$t_success &= $this->query("UPDATE `properties_description` SET `properties_name` = '$defaultNameForEmptyEnglish' WHERE (`properties_name` IS NULL OR `properties_name` = '') AND `language_id` != 2;", true);
$t_success &= $this->query("UPDATE `properties_description` SET `properties_name` = '$defaultNameForEmptyGerman' WHERE (`properties_name` IS NULL OR `properties_name` = '') AND `language_id` = 2;", true);
$t_success &= $this->query("UPDATE `properties_description` SET `properties_admin_name` = `properties_name` WHERE `properties_admin_name` IS NULL OR `properties_admin_name` = '';", true);
$t_success &= $this->query("UPDATE `properties_values` SET `display_image` = '' WHERE `display_image` IS NULL", true);

//  ==============================================
//  Repairing previously valid option value data
//  that will now cause an exception
//  ==============================================
$defaultNameForEmptyEnglish = 'Unnamed option value';     // is also used for every other language that is not german
$defaultNameForEmptyGerman  = 'Unbenannter Options Wert';

$t_success &= $this->query("UPDATE `properties_values_description` SET `values_name` = '$defaultNameForEmptyEnglish' WHERE (`values_name` IS NULL OR `values_name` = '') AND `language_id` != 2;", true);
$t_success &= $this->query("UPDATE `properties_values_description` SET `values_name` = '$defaultNameForEmptyGerman' WHERE (`values_name` IS NULL OR `values_name` = '') AND `language_id` = 2;", true);