<?php
/* --------------------------------------------------------------
   dependent.inc.php 2022-03-01
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2022 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------*/

/** @var DatabaseModel $this */
/** @var bool $t_success */

/**
 * Restore the `options_id` column in the `products_properties_combis_values` database table,
 * which might have been destroyed when importing products via CSV import.
 */
if ($this->columnExists('products_properties_combis_values', 'options_id')) {
    $query = 'UPDATE `products_properties_combis_values` `ppcv` '
             . 'JOIN `properties_values` `pv` ON  `ppcv`.`properties_values_id` = `pv`.`properties_values_id` '
             . 'SET `ppcv`.`options_id` = `pv`.`properties_id`;';
    
    $t_success &= is_numeric($this->query($query));
}

$customerLoginAccessGroupID = $this->getAdminAccessGroupIdByIdentifier('CONTROLLER', 'CustomerLogin');
if (!$customerLoginAccessGroupID) {
    $customerAccessGroupID = $this->getAdminAccessGroupIdByIdentifier('PAGE', 'create_account.php');
    $this->addAdminAccessGroupItem($customerAccessGroupID, 'CONTROLLER', 'CustomerLogin');
}

$check = $this->table_column_exists('products', 'main_category_id');
if (!$check) {
    $query     = "ALTER TABLE `products` ADD `main_category_id` INT NOT NULL DEFAULT 0";
    $t_success &= $this->query($query);
}

// Updates the `products.main_category_id` table with the lowest product category ID according to the number of categories the product has:
// ONLY one category: the ID of the "Top Category" (ID 0) is used or the lowest.
// MORE than one category: gets the lowest category ID, excluding the ID 0 (Top Category)
$sql = "UPDATE products p,
        (SELECT
            pc.products_id,
                CASE
                    WHEN COUNT(pc.categories_id) = 1 THEN MIN(pc.categories_id)
                    ELSE (SELECT
                            MIN(categories_id)
                        FROM
                            products_to_categories pc2
                        WHERE
                            pc.products_id = pc2.products_id
                                AND pc2.categories_id > 0)
                END categories_id
        FROM
            `products_to_categories` pc
        GROUP BY pc.products_id) products_to_categories
    SET
        p.main_category_id = products_to_categories.categories_id
    WHERE
        p.products_id = products_to_categories.products_id AND
        p.main_category_id = 0;";

$t_success &= is_numeric($this->query($sql));

/**
 * Add missing admin access group items for recently added routes
 */

$categoriesGroupID = $this->getAdminAccessGroupIdByIdentifier('PAGE', 'categories.php') ? : 11;
$this->addAdminAccessGroupItem($categoriesGroupID, 'ROUTE', '/admin/api/image-lists');
$this->addAdminAccessGroupItem($categoriesGroupID, 'ROUTE', '/admin/api/options');
$this->addAdminAccessGroupItem($categoriesGroupID, 'ROUTE', '/admin/api/products');
$this->addAdminAccessGroupItem($categoriesGroupID, 'ROUTE', '/admin/api/vpe');
$this->addAdminAccessGroupItem($categoriesGroupID, 'ROUTE', '/admin/options');
$this->addAdminAccessGroupItem($categoriesGroupID, 'ROUTE', '/admin/products');
$this->addAdminAccessGroupItem($categoriesGroupID, 'ROUTE', '/api.php/v3/image-lists');
$this->addAdminAccessGroupItem($categoriesGroupID, 'ROUTE', '/api.php/v3/options');
$this->addAdminAccessGroupItem($categoriesGroupID, 'ROUTE', '/api.php/v3/products');

$statisticsGroupID = $this->getAdminAccessGroupIdByIdentifier('PAGE', 'whos_online.php') ? : 42;
$this->addAdminAccessGroupItem($statisticsGroupID, 'ROUTE', '/admin/api/statistics/overview');
$this->addAdminAccessGroupItem($statisticsGroupID, 'ROUTE', '/admin/statistics/overview');
$this->addAdminAccessGroupItem($statisticsGroupID, 'ROUTE', '/admin/dashboard/statistics');

$adminGroupId = $this->getAdminAccessGroupIdByIdentifier('ROUTE', '/admin') ? : 2;
$this->addAdminAccessGroupItem($adminGroupId, 'ROUTE', '/admin/api/user-configuration');
$this->addAdminAccessGroupItem($adminGroupId, 'ROUTE', '/admin/session/keep-alive');
$this->addAdminAccessGroupItem($adminGroupId, 'ROUTE', '/admin/dashboard');

$ordersGroupId = $this->getAdminAccessGroupIdByIdentifier('CONTROLLER', 'OrdersOverview') ? : 2;
$this->addAdminAccessGroupItem($ordersGroupId, 'ROUTE', '/admin/dhlreturns');

/**
 * Setting writing and deleting permissions for "Admin Web UI" and "REST API" admin access groups,
 * if admin access role has reading permission. Configuring these permissions was added with this version
 * (it wasn't possible before).
 */
$apiGroupId   = $this->getAdminAccessGroupIdByIdentifier('ROUTE', '/api.php/v3') ? : 1;
$adminGroupId = $this->getAdminAccessGroupIdByIdentifier('ROUTE', '/admin') ? : 2;
$query        = "UPDATE `admin_access_permissions`
                 SET `writing_granted` = `reading_granted`,
                      `deleting_granted` = `reading_granted`
                 WHERE `admin_access_group_id` = " . $apiGroupId . " OR
                       `admin_access_group_id` = " . $adminGroupId;
$t_success    &= $this->query($query, true);

if ($this->columnExists('withdrawals', 'customer_gender')) {
    
    $changeDefaultColumnValueQuery = "ALTER TABLE `withdrawals` CHANGE `customer_gender` `customer_gender` VARCHAR(1) CHARACTER SET utf8 COLLATE utf8_general_ci NOT NULL DEFAULT '';";
    $t_success &= $this->query($changeDefaultColumnValueQuery, true);
    
    $updateDiverseValueInWithdrawals = "UPDATE `withdrawals` SET `customer_gender`='o' WHERE `customer_gender`='d';";
    $t_success &= is_numeric($this->query($updateDiverseValueInWithdrawals));
}

if (!file_exists(DIR_FS_CATALOG . 'ext/mailhive/common/local/gbo.php')) {
    $mailbeezFileContent = <<<'FILE'
<?php
/*
  MailBeez Automatic Trigger Email Campaigns
  http://www.mailbeez.com

  Copyright (c) 2010 - 2015 MailBeez

  inspired and in parts based on
  Copyright (c) 2003 osCommerce

  Released under the GNU General Public License (Version 2)
  [http://www.gnu.org/licenses/gpl-2.0.html]

 */

define('MH_ID', 'gbo');

FILE;
    
    @mkdir(DIR_FS_CATALOG . 'ext/mailhive/common');
    @mkdir(DIR_FS_CATALOG . 'ext/mailhive/common/local');
    @file_put_contents(DIR_FS_CATALOG . 'ext/mailhive/common/local/gbo.php', $mailbeezFileContent);
    clearstatcache(true);
    if (!is_writable(DIR_FS_CATALOG . 'ext/mailhive/common/local/gbo.php')) {
        @chmod(DIR_FS_CATALOG . 'ext/mailhive/common/local/gbo.php', 0777);
    }
}

if (!file_exists(DIR_FS_CATALOG . 'ext/mailhive/common/local/index.html')) {
    @file_put_contents(DIR_FS_CATALOG . 'ext/mailhive/common/local/index.html', '<html></html>');
    clearstatcache(true);
    if (!is_writable(DIR_FS_CATALOG . 'ext/mailhive/common/local/index.html')) {
        @chmod(DIR_FS_CATALOG . 'ext/mailhive/common/local/index.html', 0777);
    }
}
